=begin

 info.rb - Load Locale::Info::Language and Locale::Info::Region.

 Copyright (C) 2008  Masao Mutoh
 
 $Id: info.rb 27 2008-12-03 15:06:50Z mutoh $
=end

require 'locale/info/language'
require 'locale/info/region'

